#-----------------------------------------------------------------------------
#
#   PyWeek 10 - Playing screen
#
#-----------------------------------------------------------------------------

from __future__ import division
from time import time
from OpenGL import GL
from pygame import Rect, K_LEFT, K_RIGHT, K_UP, K_DOWN, K_s
from albow.widget import Widget
from albow.controls import Label
from albow.layout import Row, Column
from albow.opengl import GLOrtho
from humerus.main_commands import MainCommandScreen
from game import game
from sprite import LOW_JUMP, HIGH_JUMP
import perf

def init_texturing():
	if 1:
		GL.glEnable(GL.GL_TEXTURE_2D)
		GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE)
		GL.glBindTexture(GL.GL_TEXTURE_2D, game.jelly_texture)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
		GL.glColor4f(0, 0, 0, 0)
	else:
		GL.glColor4f(1, 0, 0, 1)

def draw_grid(grid):
	isize, jsize = grid.dimensions
	if 1:
		if 1:
			GL.glVertexPointerd(grid.vertices)
		else:
			s = grid.vertices.tostring()
			GL.glVertexPointer(2, GL.GL_DOUBLE, 0, s)
	if 1:
		if 1:
			GL.glTexCoordPointerd(grid.tex_coords)
		else:
			s = grid.tex_coords.tostring()
			GL.glTexCoordPointer(2, GL.GL_DOUBLE, 0, s)
	if 1:
		GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
		GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
		if 1:
			GL.glDrawElementsui(GL.GL_QUADS, grid.indices)
		else:
			GL.glBegin(GL.GL_QUADS)
			for i in xrange(len(grid.indices)):
				k = grid.indices[i]
				if 0:
					GL.glArrayElement(k)
				else:
					GL.glTexCoord2fv(grid.tex_coords[k])
					GL.glVertex2fv(grid.vertices[k])
			GL.glEnd()
		GL.glDisableClientState(GL.GL_VERTEX_ARRAY)
		GL.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY)
	else:
		GL.glBegin(GL.GL_QUADS)
		us = 1 / isize
		ut = 1 / jsize
		P = grid.P
		def gl_grid_point(i, j):
			GL.glTexCoord2f(i * us, j * ut)
			GL.glVertex2fv(P[i, j])
		for j in xrange(jsize):
			for i in xrange(isize):
				gl_grid_point(i, j)
				gl_grid_point(i+1, j)
				gl_grid_point(i+1, j+1)
				gl_grid_point(i, j+1)
		GL.glEnd()

def draw_sprites(sprites, grid, yrange):
	for sprite in sprites:
		sprite.draw(game, yrange)

#-----------------------------------------------------------------------------

class JellyView(GLOrtho):

	floor_height = 5 * 32
	lower_scroll_threshold = floor_height - floor_height // 2
	upper_scroll_threshold = 2 * floor_height + floor_height // 2
	scroll_increment = floor_height
	scroll_rate = 8

	def __init__(self, width, height):
		GLOrtho.__init__(self, Rect(0, 0, width, height), 0, width, 0, height)
	
	def gl_draw(self):
		perf.begin("Render")
		from game import grid_cell_size
		state = game.state
		grid = state.grid
		isize, jsize = grid.dimensions
		dx = 0.5 * (self.width - isize * grid_cell_size)
		dy = state.scroll_offset
		GL.glTranslatef(dx, -dy, 0.0)
		GL.glEnable(GL.GL_BLEND)
		GL.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA)
		init_texturing()
		draw_grid(grid)
		perf.begin("draw sprites")
		y0 = game.state.scroll_offset
		y1 = y0 + self.height
		draw_sprites(state.sprites, grid, (y0, y1))
		perf.end("draw sprites")
		perf.end("Render")
	
	def begin_frame(self):
		state = game.state
		py = game.player.position[1]
		tso = state.target_scroll_offset
		spy = py - tso
		si = self.scroll_increment
		if spy > self.upper_scroll_threshold:
			state.target_scroll_offset = tso + si
		elif tso > 0 and spy < self.lower_scroll_threshold:
			state.target_scroll_offset = max(0, tso - si)
		#print "JellyView.begin_frame: target_scroll_offset =", state.target_scroll_offset ###
		if state.scroll_offset < state.target_scroll_offset:
			state.scroll_offset += self.scroll_rate
		elif state.scroll_offset > state.target_scroll_offset:
			state.scroll_offset -= self.scroll_rate

#-----------------------------------------------------------------------------

class StatusLabel(Label):

	fg_color = (0xc0, 0xc0, 0xc0)

#-----------------------------------------------------------------------------

class SonicChargeView(Widget):

	fg_color = (99, 207, 19)
	border_color = (99, 207, 19)
	border_width = 2
	margin = 2
	
	def __init__(self, width, height):
		Widget.__init__(self, Rect(0, 0, width, height))

	def draw(self, surf):
		player = game.player
		f = player.sonic_charge / player.max_sonic_charge
		r = self.get_margin_rect()
		r.width *= f
		surf.fill(self.fg_color, r)

#-----------------------------------------------------------------------------

class PlayScreen(MainCommandScreen):

	is_gl_container = True
	
	sonic_mode = False

	def __init__(self, *args, **kwds):
		MainCommandScreen.__init__(self, *args, **kwds)
		scv = SonicChargeView(300, 20)
		status = Row([
			StatusLabel("Sonic Screwdriver Charge", fg_color = (99, 207, 19)),
			scv,
		], margin = 5)
		jv = self.jelly_view = JellyView(self.width, self.height - status.height)
		status.top = jv.bottom
		self.add(status)
		self.add(jv)
	
	def begin_frame(self):
		perf.begin_frame()
		self.jelly_view.begin_frame()
		self.invalidate()

	def key_down(self, e):
		k = e.key
		if k == K_LEFT:
			if self.sonic_mode:
				game.sonic_left()
			else:
				game.start_walking_left()
		elif k == K_RIGHT:
			if self.sonic_mode:
				game.sonic_right()
			else:
				game.start_walking_right()
		elif k == K_UP:
			if self.sonic_mode:
				game.sonic_up()
			else:
				if e.shift:
					game.jump(HIGH_JUMP)
				else:
					game.jump(LOW_JUMP)
		elif k == K_DOWN:
			if self.sonic_mode:
				game.sonic_down()
			else:
				game.face_front()
		elif k == K_s:
			self.sonic_mode = True
		else:
			MainCommandScreen.key_down(self, e)

	def key_up(self, e):
		game.stop_sonic()
		k = e.key
		if k == K_LEFT:
			game.stop_walking_left()
		elif k == K_RIGHT:
			game.stop_walking_right()
		elif k == K_UP:
			game.cancel_jump()
		elif k == K_s:
			self.sonic_mode = False
		else:
			MainCommandScreen.key_up(self, e)

